program RecordStream;

{$modeswitch advancedrecords}

uses
  SysUtils, Classes;

type
  TStudentRecord = record
    s_name: string;
    s_addr: string;
    s_batchcode: string;
  end;
  TStudents = array of TStudentRecord;

procedure WriteIntegerToStream(Stream: TStream; const Value: Integer);
begin
  Stream.WriteBuffer(Value, SizeOf(Value));
end;

procedure WriteStringToStream(Stream: TStream; const Value: string);
var
  S: WideString;
  Len: Integer;
begin
  S := WideString(Value);
  Len := Length(S) * SizeOf(WideChar);
  Stream.WriteBuffer(Len, SizeOf(Len));
  Stream.WriteBuffer(PWideChar(S)^, Len);
end;

function ReadIntegerFromStream(Stream: TStream): Integer;
begin
  Stream.ReadBuffer(Result, SizeOf(Result));
end;

function ReadStringFromStream(Stream: TStream): string;
var
  S: WideString;
  Len: Integer;
begin
  Stream.ReadBuffer(Len, SizeOf(Len));
  SetLength(S, Len div SizeOf(WideChar));
  Stream.ReadBuffer(PWideChar(S)^, Len);
  Result := string(S);
end;

procedure SaveToStream(const AStudents: TStudents);
var
  SavingStream: TFileStream;
  i: Integer;
begin
  SavingStream := TFileStream.Create(ExtractFilePath(ParamStr(0)) + 'students.dat', fmCreate or fmOpenWrite or fmShareDenyWrite);
  try
    WriteIntegerToStream(SavingStream, Length(AStudents));
    for i := Low(AStudents) to High(AStudents) do
      begin
        WriteStringToStream(SavingStream, AStudents[i].s_name);
        WriteStringToStream(SavingStream, AStudents[i].s_addr);
        WriteStringToStream(SavingStream, AStudents[i].s_batchcode);
      end;
  finally
    SavingStream.Free;
  end;
end;

function LoadFromStream: TStudents;
var
  LoadingStream: TFileStream;
  i, Len: Integer;
begin
  LoadingStream := TFileStream.Create(ExtractFilePath(ParamStr(0)) + 'students.dat', fmOpenRead or fmShareDenyWrite);
  try
    Len := ReadIntegerFromStream(LoadingStream);
    SetLength(Result, Len);
    for i := 0 to Pred(Len) do
      begin
        Result[i].s_name      := ReadStringFromStream(LoadingStream);
        Result[i].s_addr      := ReadStringFromStream(LoadingStream);
        Result[i].s_batchcode := ReadStringFromStream(LoadingStream);
      end;
  finally
    LoadingStream.Free;
  end;
end;

// demonstration:
var
  Students: TStudents;
  i: Integer;
begin
  Randomize;
  // fill array with dummy data
  SetLength(Students, Random(100) + 1);
  for i := Low(Students) to High(Students) do
    begin
      Students[i].s_addr := 'Student.s_addr ' + IntToStr(i);
      Students[i].s_batchcode := 'Student.s_batchcode ' + IntToStr(i);
      Students[i].s_name := 'Student.s_name ' + IntToStr(i);
    end;
  // save it via stream
  SaveToStream(Students);
  // flush the records
  SetLength(Students, 0);
  // fill data from stream into array
  Students := LoadFromStream;
  // show it
  for i := Low(Students) to High(Students) do
    begin
      WriteLn(Students[i].s_addr);
      WriteLn(Students[i].s_batchcode);
      WriteLn(Students[i].s_name);
    end;
  ReadLn;
end.

